@@ -3,10 +3,11 @@ use crate::{utils::BTreeBijection, Quad, Triple};
 use derivative::Derivative;
 use rdf_types::{AsTerm, IntoTerm};
 use std::collections::{BTreeMap, BTreeSet};
+use std::fmt;
 use std::hash::Hash;
 
 /// Graph implementation based on `BTreeMap` and `BTreeSet`.
-#[derive(Debug, Derivative)]
+#[derive(Derivative)]
 #[derivative(PartialEq(bound = "S: Ord, P: Ord, O: Ord"))]
 #[derivative(Eq(bound = "S: Ord, P: Ord, O: Ord"))]
 #[derivative(PartialOrd(bound = "S: Ord, P: Ord, O: Ord"))]
@@ -24,6 +25,22 @@ impl<S, P, O> BTreeGraph<S, P, O> {
 	}
 }
 
+impl<S: fmt::Debug, P: fmt::Debug, O: fmt::Debug> fmt::Debug for BTreeGraph<S, P, O> {
+	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+		write!(f, "{{")?;
+
+		for (i, rdf_types::Triple(s, p, o)) in self.triples().enumerate() {
+			if i > 0 {
+				write!(f, ",")?;
+			}
+
+			write!(f, " {:?} {:?} {:?}", s, p, o)?;
+		}
+
+		write!(f, "  }}")
+	}
+}
+
 impl<S: Ord, P: Ord, O: Ord> BTreeGraph<S, P, O> {
 	/// Create a new `BTreeGraph` from another graph by consuming its triples.
 	pub fn from_graph<G: crate::SizedGraph<Subject = S, Predicate = P, Object = O>>(
@@ -552,7 +569,7 @@ impl<S: Ord, P: Ord, O: Ord> std::iter::Extend<Triple<S, P, O>> for BTreeGraph<S
 	}
 }
 
-#[derive(Debug, Derivative)]
+#[derive(Derivative)]
 #[derivative(PartialEq(bound = "S: Ord, P: Ord, O: Ord, G: Ord"))]
 #[derivative(Eq(bound = "S: Ord, P: Ord, O: Ord, G: Ord"))]
 #[derivative(PartialOrd(bound = "S: Ord, P: Ord, O: Ord, G: Ord"))]
@@ -816,6 +833,27 @@ impl<S: Ord, P: Ord, O: Ord, G: Ord> crate::Dataset for BTreeDataset<S, P, O, G>
 	}
 }
 
+impl<S: fmt::Debug, P: fmt::Debug, O: fmt::Debug, G: fmt::Debug> fmt::Debug
+	for BTreeDataset<S, P, O, G>
+{
+	fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+		write!(f, "{{")?;
+
+		for (i, rdf_types::Quad(s, p, o, g)) in self.quads().enumerate() {
+			if i > 0 {
+				write!(f, ",")?;
+			}
+
+			match g {
+				Some(g) => write!(f, " {:?} {:?} {:?} {:?}", s, p, o, g)?,
+				None => write!(f, " {:?} {:?} {:?}", s, p, o)?,
+			}
+		}
+
+		write!(f, "  }}")
+	}
+}
+
 pub struct Graphs<'a, S, P, O, G> {
 	default: Option<&'a BTreeGraph<S, P, O>>,
 	it: std::collections::btree_map::Iter<'a, G, BTreeGraph<S, P, O>>,
